/*
 * Author: Jake G Email: jakegoodwin@gorge.works
 * FileName: main.c
 * Date: 2025
 * Descripton: Program for Atiny13A controllers
 */

#ifndef MCU
#define MCU atiny13a
#endif

#ifndef F_CPU
#define F_CPU 4800000UL
#endif

#include "stdio.h"

#include <avr/io.h>
#include <avr/eeprom.h>
#include <util/delay.h>


//#############################
//Globals
//#############################


//Debounce check number.
#define MAX_CHECKS 10

#define LED_PORT (1<<5) 
#define LED_PIN PB5
#define BLINK_DELAY_MS 5000

#define PWM_PIN1 PB0  // Pin 5 - Motor PWM 1
#define PWM_PIN2 PB1  // Pin 6 - Motor PWM 2
#define ADC_PIN PB3   // Pin 2 - Fader position reading
#define BUTTON_PIN PB4 // Pin 3 - Button input

#define LONG_PRESS_TIME 1000  // 1 second hold
#define SHORT_PRESS_TIME 50   // Debounce time

uint8_t saved_position EEMEM;  // EEPROM storage for fader position

//#############################
//Prototypes
//#############################

void led_blink(void);

void setup(void);
uint16_t readADC(void);
void motorCoast(void);
void motorMove(int direction);
uint8_t readButton(void);


//#############################
//MAIN
//#############################

int main(int argc, char **argv)
{ 

    while(1) {
        uint16_t fader_pos = readADC() >> 2; // Scale 10-bit ADC to 8-bit (0-255)
        uint8_t buttonState = readButton();
        
        if (buttonState == 2) { // Long press - Save position
            eeprom_update_byte(&saved_position, fader_pos);
        } 
        else if (buttonState == 1) { // Short press - Move to saved position
            uint8_t target_pos = eeprom_read_byte(&saved_position);
            if (fader_pos < target_pos - 2) motorMove(1); // Move up
            else if (fader_pos > target_pos + 2) motorMove(-1); // Move down
            else motorCoast(); // Stop when reached position
        } 
        else {
            motorCoast(); // Default to coast mode
        }

        led_blink();
         
    }
    
    return 0;
}


//#############################
//FUNCTIONS
//#############################


/*
 * ############################
 * DEBOUNCING CODE 
 * ############################
 */ 

/*
 * INPUT: The port to check.
 * OUTPUT: None
 * DESCRIPTION: Updates the global debounced state. This function
 * should be called in a ISR a set rate using the hardware timer.
 */
static inline void debounce_switch() {
    uint8_t i, j;
    db_state[idx] = PINB & 0xFF;
    idx+=1;
    j = 0xff;    
    /*Loop through a number of checks*/
    for(i = 0; i < MAX_CHECKS; i++) {
        j = j & db_state[i];
    }
    debounced_state = j;

    if(idx >= MAX_CHECKS) {
        idx = 0;
    }
}



/*Setup the timer0 on the AVR*/
static inline void init_timer0() { 
    /*Zero out the tick_count var*/
    tick_count = 0;

    /*config to normal mode.*/
    TCCR0A = 0x00;  //stop timer
    TCCR0B = 0x00;  //zero timer

    /*set prescaler*/
    //Set to div64
    TCCR0B |= (1<<CS01)|(1<<CS00);

    
    /*Enable global interrupts*/
    sei();

    /*Enabling timer0 interrupt*/
    TCNT0 = 0;
    TIMSK0 |= (1<<TOIE0);
}


//ISR(TIMER0_OVF_vect)

/*The interrupt service routine for the timer0*/
ISR(TIM0_OVF_vect)
{
    /*Disable global interrupts*/
    cli();

    /*Check the state of the switches*/
    debounce_switch();

    /*Update the tick_count*/
    if(btn1.timer_enabled && btn1.pressed_ticks <= UINT8_MAX){
        btn1.pressed_ticks += 1;
    }

    if(btn2.timer_enabled && btn2.pressed_ticks <= UINT8_MAX){
        btn2.pressed_ticks += 1;
    }


    /*Re-Enable global interrupts*/
    sei();
}
